/*
    This file is part of Blobtrixd
    Copyright (C) 2004-2005 Markus Kettunen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "block.h"

#include <stdlib.h>

char blockgroup[7][4][4][4]={
{
{{0,0,0,0}, {1,1,1,0}, {0,1,0,0}, {0,0,0,0}}, // T
{{0,1,0,0}, {1,1,0,0}, {0,1,0,0}, {0,0,0,0}},
{{0,1,0,0}, {1,1,1,0}, {0,0,0,0}, {0,0,0,0}},
{{0,1,0,0}, {0,1,1,0}, {0,1,0,0}, {0,0,0,0}}
},
{
{{0,2,0,0}, {2,2,0,0}, {2,0,0,0}, {0,0,0,0}}, // Z
{{0,0,0,0}, {2,2,0,0}, {0,2,2,0}, {0,0,0,0}},
{{0,2,0,0}, {2,2,0,0}, {2,0,0,0}, {0,0,0,0}},
{{0,0,0,0}, {2,2,0,0}, {0,2,2,0}, {0,0,0,0}}
},
{
{{3,0,0,0}, {3,3,0,0}, {0,3,0,0}, {0,0,0,0}}, // S
{{0,0,0,0}, {0,3,3,0}, {3,3,0,0}, {0,0,0,0}},
{{3,0,0,0}, {3,3,0,0}, {0,3,0,0}, {0,0,0,0}},
{{0,0,0,0}, {0,3,3,0}, {3,3,0,0}, {0,0,0,0}}
},
{
{{4,4,0,0}, {0,4,0,0}, {0,4,0,0}, {0,0,0,0}}, // L
{{0,0,4,0}, {4,4,4,0}, {0,0,0,0}, {0,0,0,0}},
{{0,4,0,0}, {0,4,0,0}, {0,4,4,0}, {0,0,0,0}},
{{0,0,0,0}, {4,4,4,0}, {4,0,0,0}, {0,0,0,0}}
},
{
{{0,5,5,0}, {0,5,0,0}, {0,5,0,0}, {0,0,0,0}}, // J
{{0,0,0,0}, {5,5,5,0}, {0,0,5,0}, {0,0,0,0}},
{{0,5,0,0}, {0,5,0,0}, {5,5,0,0}, {0,0,0,0}},
{{5,0,0,0}, {5,5,5,0}, {0,0,0,0}, {0,0,0,0}}
},
{
{{0,6,0,0}, {0,6,0,0}, {0,6,0,0}, {0,6,0,0}}, // I
{{0,0,0,0}, {6,6,6,6}, {0,0,0,0}, {0,0,0,0}},
{{0,6,0,0}, {0,6,0,0}, {0,6,0,0}, {0,6,0,0}},
{{0,0,0,0}, {6,6,6,6}, {0,0,0,0}, {0,0,0,0}}
},
{
{{0,0,0,0}, {0,7,7,0}, {0,7,7,0}, {0,0,0,0}}, // O
{{0,0,0,0}, {0,7,7,0}, {0,7,7,0}, {0,0,0,0}},
{{0,0,0,0}, {0,7,7,0}, {0,7,7,0}, {0,0,0,0}},
{{0,0,0,0}, {0,7,7,0}, {0,7,7,0}, {0,0,0,0}}
}};

// T, Z, S, L, J, I, O

bool block::IsNonzero(int b, int r, int x, int y) {
	if (blockgroup[b][r][y][x]) return true;
	return false;
}
char block::GetColour(int b, int r, int x, int y) {
	return blockgroup[b][r][y][x];
}

int block::GetMinX(int type, int angle) {
	if (type<0 || type>7 || angle < 0 || angle > 3) return -1;

	for (int cx=0; cx<4; cx++) {
		for (int cy=0; cy<4; cy++) {
			if ( IsNonzero(type-1, angle, cx, cy) ) return cx;
		}
	}
	return -1;
}
int block::GetMaxX(int type, int angle) {
	if (type<0 || type>7 || angle < 0 || angle > 3) return -1;

	for (int cx=3; cx>=0; cx--) {
		for (int cy=0; cy<4; cy++) {
			if ( IsNonzero(type-1, angle, cx, cy) ) return cx;
		}
	}
	return -1;
}
int block::GetMinY(int type, int angle) {
	if (type<0 || type>7 || angle < 0 || angle > 3) return -1;

	for (int cy=0; cy<4; cy++) {
		for (int cx=0; cx<4; cx++) {
			if ( IsNonzero(type-1, angle, cx, cy) ) return cy;
		}
	}
	return -1;
}
int block::GetMaxY(int type, int angle) {
	if (type<0 || type>7 || angle < 0 || angle > 3) return -1;

	for (int cy=3; cy>=0; cy--) {
		for (int cx=0; cx<0; cx++) {
			if ( IsNonzero(type-1, angle, cx, cy) ) return cy;
		}
	}
	return -1;
}

int block::GetMiddleX(int type, int angle) {
	int maxx = GetMaxX(type, angle);
	int minx = GetMinX(type, angle);

	return 19*minx + (int)( (maxx-minx)*19/2 );
}

int block::GetMiddleY(int type, int angle) {
	int maxy = GetMaxY(type, angle);
	int miny = GetMinY(type, angle);

	return 19*miny + (int)( (maxy-miny)*19/2 );
}
