/*
    This file is part of Blobtrixd
    Copyright (C) 2004-2005 Markus Kettunen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "server.h"

#include "net.h"
#include "variable.h"
#include "block.h"

#include <unistd.h>

extern block Block;

int scoretable[5]={0, 100, 225, 375, 500};

server::server() {
	requirepassword=false;

/*	memset(blocks, 0, XSIZE*YSIZE);
	for (int j=0; j<2; j++) for (int i=0; i<5; i++) players[j].blocks[i]=NULL;
	newblocktime=1000;*/

}


void server::ContactMetaserver() {
	char data[256];
	sprintf (data, "LISTME %d %d %d %d %d %c1.3.2%c%s%c", Config.dedicated, Config.lightning, 
Config.gametime, PlayersJoined(), Config.followers, 0x01, 0x01, Config.infoline, 0x01);
	net::SendUDPpacket_unbound(sock, metaaddress, data);
}

void server::serve(char *filename) {
	Config.load(filename);

	sock = SDLNet_UDP_Open( Config.port );
	if (!sock) {
		cout << "Couldn't open socket! Port perhaps in use!\n";
		exit(1);
	}
	cout << "Started listening port " << Config.port << ".\n";

	if (daemon(0,0)==-1) {
		cout << "Couldn't fall in background!\n";
	}


	UDPpacket *packet = SDLNet_AllocPacket(65535);

	if (Config.contactmetaserver) {
		SDLNet_ResolveHost(&metaaddress, "shoebox.ath.cx", 8855);
		ContactMetaserver();
	}

	Uint32 lastping=SDL_GetTicks();
	Uint32 nextupdate=SDL_GetTicks();

	Uint32 metaupdate=SDL_GetTicks()+600000;

	Uint32 physframetime = 10; // (1000/100) -- 100 phys frames per second
	gametime = Min2Frame(Config.gametime);

	Uint32 pingnumber=0;


	NewGame();

	while (1) {
		SDL_Delay(5);


		// ping players and check ping timeout

		if (SDL_GetTicks()-lastping>=100) { // check every 1/10th second)

			// ping

			pingnumber++;
			char data[100];
			sprintf (data, "PING %d", pingnumber);

			for (int i=0; i<2; i++) {
				if (players[i].free==false) {
					net::SendUDPpacket(sock, i, data);
				}
			}

			if (Config.followers) {
				// ping followers
				for (int i=0; i<MAXFOLLOWERS; i++) {
					if (followers[i].free==false) {
						net::SendUDPpacket(sock, i+2, data);
					}
				}
			}

			lastping=SDL_GetTicks();


			// ping timeouts
			for (int i=0; i<2; i++) {
				if (players[i].free==false) {
					if (SDL_GetTicks()-players[i].lastping() >= (Uint32)Config.timeout) {
						players[i].free=true;
						SDLNet_UDP_Unbind(sock, i);

						if (players[1-i].free==false) {
							char data[]="YOUAREALONE";
							net::SendUDPpacket(sock, 1-i, data);

							if (Config.followers) {
								for (int j=0; j<MAXFOLLOWERS; j++) {
									if (followers[j].free==false) net::SendUDPpacket(sock, j+2, data);
								}
							}

						} else {

							if (Config.followers) {
								char data[]="YOUARENONE";
								for (int j=0; j<MAXFOLLOWERS; j++) {
									if (followers[j].free==false) net::SendUDPpacket(sock, j+2, data);
								}
							}
						
							NewGame();
						}

						if (Config.contactmetaserver) ContactMetaserver();
					}
				}
			}

			if (Config.followers) {
				// ping timeout followers
				for (int i=0; i<MAXFOLLOWERS; i++) {
					if (followers[i].free==false) {
						if (SDL_GetTicks()-followers[i].lastping() >= (Uint32)Config.timeout) {
							followers[i].free=true;
							SDLNet_UDP_Unbind(sock, i+2);

							char data[512];
							sprintf (data, "CHAT Quit: %s", followers[i].GetNick());
							RawSendAll(data);
						}
					}
				}
			}
		}


		if (Config.contactmetaserver)
			if (SDL_GetTicks() > metaupdate) {
			metaupdate+=60000;
			ContactMetaserver();
		}


		if (SDLNet_UDP_Recv(sock, packet)) {

			if ( variable::str_splitcompare( (char*)packet->data, 0, "PING") ) {
				char data[]="PONG";
				net::SendUDPpacket_unbound(sock, packet->address, data);
			}

			if ( variable::str_splitcompare( (char*)packet->data, 0, "CONNECT") ) {
				// check if the player can join and do whatever must be done


				if (!( RecognizePlayer(packet->address)>=0 || RecognizeFollower(packet->address)>=0) ) {

					int firstfree = FirstFreePlayerslot();
					if (firstfree>=0) {

						// Get the nick after "connect"
						char nick[21];
						char *nickname = variable::str_split( (char*)packet->data, ' ', 1);
						if (nickname) {
							strncpy (nick, nickname, 20);
							delete[] nickname;
	
							if (CheckNickname(nick)) {
	
								bool unique=true;
								for (int i=0; i<2; i++)
									if (players[i].free==false && strcmp(nick, players[i].GetNick())==0) unique=false;
								for (int i=0; i<MAXFOLLOWERS; i++)
									if ( followers[i].free==false && strcmp(nick, players[i].GetNick())==0) unique=false;
		
	
								if (unique) {
	
									if ( (requirepassword && variable::str_splitcompare( (char*)packet->data, 2, password) ) || (!requirepassword) ) {
	
										// correct password or no password
										// so now we're ready except the bans.
										players[firstfree].free=false;
										players[firstfree].SetNick(nick);
										players[firstfree].pinged();
										players[firstfree].address = packet->address;
										SDLNet_UDP_Bind(sock, firstfree, &packet->address);
		
										char data[]="CONNECTIONACCEPTED";
										net::SendUDPpacket(sock, firstfree, data);
		
										// getting ready for the competition!
										// selecting the slots
										if (PlayersJoined()==2) {
											nextupdate = SDL_GetTicks();
											SendEverything();
										} else {
											char data[]="YOUAREALONE";
											net::SendUDPpacket(sock, 1-firstfree, data);
										}
		
										if (Config.contactmetaserver) ContactMetaserver();
									} else {
										char data[]="CONNECTIONDENIED Wrong password!";
										net::SendUDPpacket_unbound(sock, packet->address, data);
									}
								} else {
									char data[]="CONNECTIONDENIED Nickname in use!";
									net::SendUDPpacket_unbound(sock, packet->address, data);
								}
							} else {
								char data[]="CONNECTIONDENIED Bad nickname";
								net::SendUDPpacket_unbound(sock, packet->address, data);
							}
						} else {
							char data[]="CONNECTIONDENIED Please enter a nickname";
							net::SendUDPpacket_unbound(sock, packet->address, data);
						}
					} else {
						char data[]="CONNECTIONDENIED We're full";
						net::SendUDPpacket_unbound(sock, packet->address, data);
					}
				} else {
					char data[]="CONNECTIONDENIED Already logged in.";
					net::SendUDPpacket_unbound(sock, packet->address, data);
				}	


			}


			if ( variable::str_splitcompare( (char*)packet->data, 0, "FOLLOW") ) {
				// check if the follower can join and do whatever must be done

				if (Config.followers) { // if followers are allowed

					if (!( RecognizePlayer(packet->address)>=0 || RecognizeFollower(packet->address))>=0 ) {

						if (PlayersJoined()==2) {
							int firstfree = FirstFreeFollowerslot();
							if (firstfree>=0) {
								if ( (requirepassword && variable::str_splitcompare( (char*)packet->data, 2, 
											password) ) || (!requirepassword) ) {
									// Get the nick after "follow"
									char nick[21];
									char *nickname = variable::str_split( (char*)packet->data, ' ', 1);
		
									if (nickname) {
										strncpy (nick, nickname, 20);
										delete[] nickname;
	
										if (CheckNickname(nick)) {
											bool unique=true;
											for (int i=0; i<2; i++) if (players[i].free==false && strcmp(nick, players[i].GetNick())==0) unique=false;
											for (int i=0; i<MAXFOLLOWERS; i++)
												if ( followers[i].free==false && strcmp(nick, players[i].GetNick())==0) unique=false;
		
											if (unique) {
											// correct password or no password
											// so now we're ready except the bans. (which do not exist.. as doesn't the 
											// password ;)
					
												followers[firstfree].SetNick(nick);
		
												followers[firstfree].free=false;
												followers[firstfree].pinged();
												followers[firstfree].address = packet->address;
												SDLNet_UDP_Bind(sock, firstfree+2, &packet->address);
				
												char data[]="CONNECTIONACCEPTED";
												net::SendUDPpacket(sock, firstfree+2, data);
					
												nextupdate = SDL_GetTicks();
												SendEverything();
	
												char data2[512];
												sprintf (data2, "CHAT Join: %s", nick);
												RawSendAll(data2);	
											} else {
												char data[]="CONNECTIONDENIED Nickname in use!";
												net::SendUDPpacket_unbound(sock, packet->address, data);
											}
										} else {
											char data[]="CONNECTIONDENIED Illegal nickname";
											net::SendUDPpacket_unbound(sock, packet->address, data);
										}
									} else {
										char data[]="CONNECTIONDENIED Please enter a nickname";
										net::SendUDPpacket_unbound(sock, packet->address, data);
									}
	
								} else {
									char data[]="CONNECTIONDENIED Wrong password!";
									net::SendUDPpacket_unbound(sock, packet->address, data);
								}
	
							} else {
								char data[]="CONNECTIONDENIED We're full";
								net::SendUDPpacket_unbound(sock, packet->address, data);
							}
	
						} else {
							char data[]="CONNECTIONDENIED Not enough players to follow!";
							net::SendUDPpacket_unbound(sock, packet->address, data);
						}
					} else {
						char data[]="CONNECTIONDENIED Already logged in.";
						net::SendUDPpacket_unbound(sock, packet->address, data);
					}	
				} else {
					char data[]="CONNECTIONDENIED No followers!";
					net::SendUDPpacket_unbound(sock, packet->address, data);
				}
			}



			if ( variable::str_splitcompare( (char*)packet->data, 0, "PONG") ) {
				int player = RecognizePlayer(packet->address);
				if (player>=0) {
					int k = variable::str_splitgetint( (char*)packet->data, ' ', 1);
					players[player].ping=SDL_GetTicks()-lastping+(pingnumber-k)*100;
					players[player].pinged();

				} else if (Config.followers) {
					int f = RecognizeFollower(packet->address);
					if (f>=0) {
						int k = variable::str_splitgetint( (char*)packet->data, ' ', 1);
						followers[f].ping=SDL_GetTicks()-lastping+(pingnumber-k)*100;
						followers[f].pinged();
					}
				}
			}

			if ( variable::str_splitcompare( (char*)packet->data, 0, "CHAT") ) {
				if (PlayersJoined()==2) {
					int player = RecognizePlayer(packet->address);
					if (player>=0) {
						char *msg = variable::str_fullsplit( (char*)packet->data, ' ', 1);
						if (strlen(msg)>0 && strlen(msg)<480) {
							char data[512];
							sprintf (data, "CHAT <%s> %s", players[player].GetNick(), msg);

							RawSendAll(data);

						}
						if (msg) {
							delete[] msg;
						}
					} else if (Config.followers) {
						int f = RecognizeFollower(packet->address);
						if (f>=0) {
							char *msg = variable::str_fullsplit( (char*)packet->data, ' ', 1);
							if (strlen(msg)>0 && strlen(msg)<480) {
								char data[512];
								sprintf (data, "CHAT (%s) %s", followers[f].GetNick(), msg);

								RawSendAll(data);
							}
						}
					}
				}
			}


			if ( variable::str_splitcompare( (char*)packet->data, 0, "NEWGAME") ) {
				if (PlayersJoined()==2) {
					int player = RecognizePlayer(packet->address);
					if (player>=0) {
						players[player].newgame+=500;
						char data1[]="YOUNEWGAME";
						char data2[]="ENEMYNEWGAME";
						net::SendUDPpacket(sock, player, data1);
						net::SendUDPpacket(sock, 1-player, data2);

						if (Config.followers) {
							if (player==0) {
								for (int j=0; j<MAXFOLLOWERS; j++) {
									if (followers[j].free==false) net::SendUDPpacket(sock, j+2, data1);
								}
							} else {
								for (int j=0; j<MAXFOLLOWERS; j++) {
									if (followers[j].free==false) net::SendUDPpacket(sock, j+2, data2);
								}
							}
						}

					}
				}
			}

			if ( variable::str_splitcompare( (char*)packet->data, 0, "WHOAMI") ) {
				int player = RecognizePlayer(packet->address);
				if (player>=0) {
					char data[40];
					sprintf (data, "YOUARE %d", player);
					net::SendUDPpacket(sock, player, data);
				}
			}


			if (winner==-1) { // if we're below this the game is being played

				if ( variable::str_splitcompare( (char*)packet->data, 0, "SELECTIONLEFT") ||
					  variable::str_splitcompare( (char*)packet->data, 0, "SELECTIONRIGHT") ) {
					if (PlayersJoined()==2) {
					 	int i = RecognizePlayer(packet->address);
	
						if (i>=0) {
	
							// the default coordinates of the block
							int x = (int)(XSIZE/2)-2;
							int y = (1-i)*YSIZE-(1-i)*4; // 0 for p2, YSIZE-4 for p1
							int angle = 1;
							int free=-1; // the first free slot - or the slot the current block will go
		
							// delete the old block (move it back to the slot)
							if (players[i].controlling>=0) {
		
								// get the coordinates of our current block
								x = players[i].blocks[ players[i].controlling ]->GetX();
								y = players[i].blocks[ players[i].controlling ]->GetY();
								angle = players[i].blocks[ players[i].controlling ]->GetAngle();
		
								players[i].slot[ players[i].GetSelection() ] = players[i].blocks[ players[i].controlling]->GetType();
		
								DeletePlayersBlock(i, players[i].controlling);
		
								free = players[i].controlling;
								players[i].controlling=-1;
							}
		
							// adjust selection as asked
							int dir=0;
							if (variable::str_splitcompare( (char*)packet->data, 0, "SELECTIONLEFT") ) dir=-1;
							else dir=1;
		
							// skip empty slots..
		
							int tries=0;
							int sel = players[i].GetSelection();
							while (1) {
								sel+=dir;
								if (sel>4) sel-=5;
								if (sel<0) sel+=5;
		
								if (players[i].slot[sel]>0) {
									players[i].SetSelection(sel);
									break;
								} else tries++;
								if (tries>5) break;
							}
		
							// find the first free slot if we didn't replace an old one
							if (free==-1) {
								for (int j=0; (j<5 && free<0); j++) {
									if (players[i].blocks[j]==NULL) free=j;
								}
							}
		
							if (free >= 0) {
								// "get the block from the slot"
			
								if (players[i].slot[ players[i].GetSelection() ] >= 1) {
									int type = players[i].slot[ players[i].GetSelection() ];
			
									players[i].blocks[free] = new tetrablock ( x, y, type, (char*)blocks );
									players[i].blocks[free]->SetAngle(angle);
			
									while (players[i].blocks[free]->CollideHorizontalWalls(x, y)==1) x++;
									while (players[i].blocks[free]->CollideHorizontalWalls(x, y)==2) x--;
		
									players[i].blocks[free]->SetX(x);
			
									players[i].controlling=free;
									SendUpdatedPlayersBlock(i, free);
									SendUpdatedPlayersSlots(i);
									SendUpdatedPlayersSlots(1-i);

									if (Config.followers) {
										// inform followers
										char data[]="PLAYSOUND 0";
										for (int j=0; j<MAXFOLLOWERS; j++) {
											if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
										}
									}

		
								}
							}
						}
					}
				}
		
				if ( variable::str_splitcompare( (char*)packet->data, 0, "LEFT") ) {
					if (PlayersJoined()==2) {
						int player = RecognizePlayer(packet->address);
						if (player>=0) {
							if ( players[player].controlling>=0 ) {
								if ( players[player].movedelay < MAXMOVEDELAY ) {
									// forbid too fast whacking
									if (players[player].controlling>=0) {
			
										int i = players[player].controlling;
			
										players[player].movedelay+=13;
			
										int x = players[player].blocks[i]->GetX()-1;
										int y = players[player].blocks[i]->GetY();
			
										if (!players[player].blocks[i]->CollideHorizontalWalls(x, y) ) {
			
											players[player].blocks[i]->SetX(x);
			
											SendUpdatedPlayersBlock(player, i);

											if (Config.followers) {
												// inform followers
												char data[]="PLAYSOUND 0";
												for (int j=0; j<MAXFOLLOWERS; j++) {
													if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
												}
											}

			
										}
									}
								}
							}
						}
					}
				}
	
				if ( variable::str_splitcompare( (char*)packet->data, 0, "RIGHT") ) {
					if (PlayersJoined()==2) {
						int player = RecognizePlayer(packet->address);
	
						if (player>=0) {
							if ( players[player].controlling>=0 ) {
								if ( players[player].movedelay < MAXMOVEDELAY ) {
									// forbid too fast whacking
									if (players[player].controlling>=0) {
										int i = players[player].controlling;
			
										players[player].movedelay+=13;
		
										int x = players[player].blocks[i]->GetX()+1;
										int y = players[player].blocks[i]->GetY();
			
										if (!players[player].blocks[i]->CollideHorizontalWalls(x, y) ) {
											players[player].blocks[i]->SetX(x);
			
											SendUpdatedPlayersBlock(player, i);

											if (Config.followers) {
												// inform followers
												char data[]="PLAYSOUND 0";
												for (int j=0; j<MAXFOLLOWERS; j++) {
													if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
												}
											}
			
										}
									}
								}
							}
						}
					}
				}
	
				if ( variable::str_splitcompare( (char*)packet->data, 0, "ROTATE") ) {
					if (PlayersJoined()==2) {
						int player = RecognizePlayer(packet->address);
						if (player>=0) {
							if (players[player].controlling>=0) {
								int i = players[player].controlling;
		
								if (!players[player].blocks[i]->Launched()) {
									int x = players[player].blocks[i]->GetX();
									int y = players[player].blocks[i]->GetY();
		
									players[player].blocks[i]->SetAngle( players[player].blocks[i]->GetAngle()+1 );
									if (players[player].blocks[i]->GetAngle() > 3) players[player].blocks[i]->SetAngle(0);
		
									while (players[player].blocks[i]->CollideHorizontalWalls(x, y)==1) x++;
									while (players[player].blocks[i]->CollideHorizontalWalls(x, y)==2) x--;
									players[player].blocks[i]->SetX(x);
		
									SendUpdatedPlayersBlock(player, i);

									if (Config.followers) {
										// inform followers
										char data[]="PLAYSOUND 1";
										for (int j=0; j<MAXFOLLOWERS; j++) {
											if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
										}
									}

								}
							}
						}
					}
				}

				if ( variable::str_splitcompare( (char*)packet->data, 0, "SHOOT") ) {
					if (PlayersJoined()==2) {
						int player = RecognizePlayer(packet->address);
						if (player>=0) {
							if (players[player].controlling >= 0 && players[player].shootwait<=0) {
								SendEvent(player, 0);
								SendEvent(1-player, 1);


								int i = players[player].controlling;
								players[player].blocks[i]->SetSpeed( Config.droptime, 2*player-1 ); /* -1 for p1, 1 for p2 */
								players[player].controlling=-1;
		
		
								players[player].slot[ players[player].GetSelection() ] = 0;
		
								for (int j=0; j<2; j++) {
									SendUpdatedPlayersSlots(j);
								}
		
								char data1[]="YOUNOSHOOT";
								char data2[]="ENEMYNOSHOOT";
		
								net::SendUDPpacket(sock, player, data1);
								net::SendUDPpacket(sock, 1-player, data2);

								if (Config.followers) {
									// informing followers
									if (player==0) {
										for (int i=0; i<MAXFOLLOWERS; i++) {
											if (followers[i].free==false) net::SendUDPpacket(sock, 2+i, data1);
										}
									} else {
										for (int i=0; i<MAXFOLLOWERS; i++) {
											if (followers[i].free==false) net::SendUDPpacket(sock, 2+i, data2);
										}
									}
								}

								players[player].shootwait=Config.shootwait;
								noshoot=0;
							}
						}
					}
				}
	
			}


		}

		if (PlayersJoined()==2) {	


			while (SDL_GetTicks() >= nextupdate) {
				nextupdate+=physframetime;

				bool updateblockmap=false;


				for (int i=0; i<2; i++) {
					if (players[i].newgame > 0) {
						if (players[i].newgame==1) {
	
							char data1[]="YOUNONEWGAME";
							char data2[]="ENEMYNONEWGAME";

							net::SendUDPpacket(sock, i, data1);
							net::SendUDPpacket(sock, 1-i, data2);

							if (Config.followers) {
								// informing followers
								if (i==0) {
									for (int j=0; j<MAXFOLLOWERS; j++) {
										if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data1);
									}
								} else {
									for (int j=0; j<MAXFOLLOWERS; j++) {
										if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data2);
									}
								}
							}

						}
						players[i].newgame--;
					}
				}

				if (players[0].newgame>0 && players[1].newgame>0) {
					NewGame();
					for (int i=0; i<2; i++) SendUpdatedPlayersBlock(i, 0);
					for (int i=0; i<2; i++) SendUpdatedPlayersSlots(i);
				}

				
				if (winner==-1) { // if we're below this the game is being played

					// If both players are stalling, time will suddenly go faster. >:)
					// (Forbids "nja nja, you didn't win yet because I don't play." when other player can't play anymore.)
					noshoot++;
					if (noshoot >= Config.noshoottime) {
						timestep=Config.noshootboost;
						gametime-=Config.noshootboost;
					}
					else {
						timestep=1;
						gametime--;
					}

					if (gametime<=0) {
						if (players[0].score > players[1].score) {
							winner=0;
							SendWin(0);
						}
						if (players[0].score < players[1].score) {
							winner=1;
							SendWin(1);
						}
						if (players[0].score == players[1].score) {
							gametime+=Min2Frame(2);
							noshoot=0;
						}

					}

					if (timeupdate<=0) {
						timeupdate=Config.timeupdaterate;
						for (int i=0; i<2; i++) {
							SendTime(i);
						}

					} else timeupdate--;


					if (pingtimetimer<=0) {
						pingtimetimer=Config.pingtimerate;
						for (int i=0; i<2; i++) {
							SendPing(i);
						}
					} else pingtimetimer--;
	
	
					for (int i=0; i<2; i++) {
						if (players[i].movedelay >0) players[i].movedelay--;
					}
	
					for (int i=0; i<2; i++) {
						if (players[i].shootwait >=0) {
							players[i].shootwait--;
							if (players[i].shootwait==0) {
								// selection selection
	
								char data1[]="YOUCANSHOOT";
								char data2[]="ENEMYCANSHOOT";

								net::SendUDPpacket(sock, i, data1);
								net::SendUDPpacket(sock, 1-i, data2);

								if (Config.followers) {
									// informing followers
									if (i==0) {
										for (int j=0; j<MAXFOLLOWERS; j++) {
											if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data1);
										}
									} else {
										for (int j=0; j<MAXFOLLOWERS; j++) {
											if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data2);
										}
									}
								}


							}
						}
					}
	
	
					// add a new block when it's time for it
					newblocktime--;
					if (newblocktime<=0) {
						newblocktime=Config.newblocktime;
	
						for (int i=0; i<2; i++) {
	
							int tries=0;
							int sel = 0;
		
							while (1) {
								sel++;
								if (sel>4) sel-=5;
		
								if (players[i].slot[sel]==0) {
									players[i].slot[sel] = (int)(rand() / (RAND_MAX/7))+1;

									if ( sel == players[i].GetSelection() ) {
										// this is the one we have the selection on so the new block will need to become selected as it appears.

										// find the first free slot we don't replace an old one
										int free=-1;
										for (int j=0; (j<5 && free<0); j++) {
											if (players[i].blocks[j]==NULL) free=j;
										}
										if (free >= 0) {
											// "get the block from the slot"
											if (players[i].slot[ players[i].GetSelection() ] >= 1) {
												int type = players[i].slot[ players[i].GetSelection() ];

												int x = (int)(XSIZE/2)-2;
												int y = (1-i)*YSIZE-(1-i)*4; // 0 for p2, YSIZE-4 for p1
						
												players[i].blocks[free] = new tetrablock ( x, y, type, (char*)blocks );
												players[i].blocks[free]->SetAngle(1);
						
												while (players[i].blocks[free]->CollideHorizontalWalls(x, y)==1) x++;
												while (players[i].blocks[free]->CollideHorizontalWalls(x, y)==2) x--;


				
												players[i].blocks[free]->SetX(x);
						
												players[i].controlling=free;
												SendUpdatedPlayersBlock(i, free);
											}
										}
									}
									break;
								} else tries++;
								if (tries>5) break;
							}
	
						}

						for (int i=0; i<2; i++) SendUpdatedPlayersSlots(i);
					}
	
	

					// blockmap-, ja seinä- ja palikkatörmäykset (=+liikutus & rivit)
	
					// kopioi palikkakartta testausta varten
					char cmap[YSIZE][XSIZE]={{0}};
					for (int y=0; y<YSIZE; y++) {
						for (int x=0; x<XSIZE; x++) {
							if (blocks[y][x]>0) cmap[y][x]=3;
						}
					}
	
					for (int i=0; i<2; i++) {
						for (int j=0; j<5; j++) {
							if (players[i].blocks[j] != NULL) {
								if (players[i].blocks[j]->Launched() ) {
									tetrablock *tb = players[i].blocks[j];
	
									// move block
									tb->droptimer--;
									if (tb->droptimer<0) {
										tb->droptimer += tb->dropspeed;
	
										bool collision = false;
	
										// collision with blockmap
										if (tb->CollideBlocks(tb->x, tb->y + tb->dropdirection)) {
											collision=true;
											tb->Fossilize();
											tb->Fossilize((char*)cmap, i+1);
	
											delete players[i].blocks[j];
											players[i].blocks[j]=NULL;
	
											// send updated blockmaps
											SendUpdatedBlockmap();

											// inform players
											char data1[200];
											char data2[200];

											sprintf (data1, "YOURBLOCKDELETED %d", j);
											net::SendUDPpacket(sock, i, data1);

											sprintf (data2, "ENEMYBLOCKDELETED %d", j);
											net::SendUDPpacket(sock, 1-i, data2);

											if (Config.followers) {
												// informing followers
												if (i==0) {
													for (int j=0; j<MAXFOLLOWERS; j++) {
														if (followers[j].free==false) {
															net::SendUDPpacket(sock, 2+j, data1);
														}
													}
												} else {
													for (int j=0; j<MAXFOLLOWERS; j++) {
														if (followers[j].free==false) {
															net::SendUDPpacket(sock, 2+j, data2);
														}
													}
												}
											}


	
										} else {
											// collision with other blocks
	
											bool ebcollide=false;

					  						for (int k=0; (k<5 && !ebcollide); k++) {
												if (players[1-i].blocks[k]!=NULL) {

													tetrablock *eb = players[1-i].blocks[k];

													// check collision	
										 			for (int cx=0; (cx<4 && !ebcollide); cx++) {
														for (int cy=0; (cy<4 && !ebcollide); cy++)  {
															if ( Block.IsNonzero(tb->GetType()-1, tb->GetAngle(), cx, cy) ) {
																int x2 = tb->GetX() - eb->GetX() + cx;
																int y2 = tb->GetY() + tb->dropdirection - eb->GetY() + cy;
	
									 							if (x2 >=0 && x2 < 4 && y2 >= 0 && y2 < 4) {
																	if (Block.IsNonzero(eb->GetType()-1, eb->GetAngle(), x2, y2) ) {
																		// remove collided blocks
																		tb->Fossilize();
																		eb->Fossilize();
																		tb->Fossilize((char*)cmap, i+1);
																		eb->Fossilize((char*)cmap, 1-i+1);
	
																		// send updated blockmaps
																		SendUpdatedBlockmap();


																		delete players[i].blocks[j];
																		players[i].blocks[j]=NULL;
																		delete players[1-i].blocks[k];
																		players[1-i].blocks[k]=NULL;
	
																		char data1[50];
																		char data2[50];
																		char data3[50];
																		char data4[50];

																		sprintf (data1, "YOURBLOCKDELETED %d", j);
																		sprintf (data2, "ENEMYBLOCKDELETED %d", k);
																		net::SendUDPpacket(sock, i, data1);
																		net::SendUDPpacket(sock, i, data2);

																		sprintf (data3, "YOURBLOCKDELETED %d", k);
																		net::SendUDPpacket(sock, 1-i, data3);
																		sprintf (data4, "ENEMYBLOCKDELETED %d", j);
																		net::SendUDPpacket(sock, 1-i, data4);


																		if (Config.followers) {
																			// informing followers
																			if (i==0) {
																				for (int j=0; j<MAXFOLLOWERS; j++) {
																					if (followers[j].free==false) {
																						net::SendUDPpacket(sock, 2+j, data1);
																						net::SendUDPpacket(sock, 2+j, data2);
																					}
																				}
																			} else {
																				for (int j=0; j<MAXFOLLOWERS; j++) {
																					if (followers[j].free==false) {
																						net::SendUDPpacket(sock, 2+j, data3);
																						net::SendUDPpacket(sock, 2+j, data4);
																					}
																				}
																			}

																		} // config.followers

																		ebcollide=true;

																	} // block.isnonzero

																} // x2,y2, ..., rectangle check
															} // block.isnonzero
					
														} // for y
													} // for x, collision check
												} // if!=NULL
					
											} // for k
											
											if (!ebcollide) { 
												tb->SetY(tb->GetY()+tb->dropdirection);
												SendUpdatedPlayersBlock(i, j);
											}

 										}
	
									}
	
								}
							}
						}
					}
	
					// and now check the lines.
					int p1l=0, p2l=0;
					for (int y=0; y<YSIZE; ) {
						// check that the line is full
						bool full=true;
						for (int x=0; (x<XSIZE && full); x++) {
							if (cmap[y][x]<=0) full=false;
						}
	
						if (full) {

							// calculate influence power for both players for this line
							updateblockmap=true;
							int p1=0, p2=0;
							for (int x=0; x<XSIZE; x++) {
								if (cmap[y][x]==1) p1++;
								if (cmap[y][x]==2) p2++;
							}
	
	
							int dir = 1;
							if (p2>p1) dir=-1;
							if (dir==1) p1l++;
							if (dir==-1) p2l++;
							if (p1==p2) dir=(int)(rand()/(RAND_MAX+1.0)*2.0)-1;
	
							// move the blocks
	
							for (int i=y+dir; (i>0 && i<YSIZE-1); i+=dir) {
								for (int x=0; x<XSIZE; x++) {
									cmap[i-dir][x]=cmap[i][x];
									blocks[i-dir][x]=blocks[i][x];
								}
							}
	
						}
						else y++;
					}

					if (Config.lightning && p1l!=p2l) {
						int diff = p1l-p2l;
						int dstep=1;
						if (diff<0) dstep=-1;

						int count=diff;
						while (count>0) {
							for (int y=0; y<YSIZE-1; y++) {
								for (int x=0; x<XSIZE; x++) {
									blocks[y][x]=blocks[y+1][x];
								}
							}
							count--;
						}
						while (count<0) {
							for (int y=YSIZE-1; y>0; y--) {
								for (int x=0; x<XSIZE; x++) {
									blocks[y][x]=blocks[y-1][x];
								}
							}
							count++;
						}

						for (int i=0; i<2; i++) {
							for (int j=0; j<5; j++) {
								if (players[i].blocks[j] != NULL && j != players[i].controlling) {
									int y = players[i].blocks[j]->GetY() - diff;
									if (y<2) y=2;
									if (y>YSIZE-6) y=YSIZE-6;
									players[i].blocks[j]->SetY( y );
								}
							}

						}

					}

	
					if (p1l>0 || p2l>0) {
						players[0].score+=scoretable[p1l];
						players[1].score+=scoretable[p2l];
						players[0].lines+=p1l;
						players[1].lines+=p2l;
						
	
						for (int i=0; i<2; i++) SendScores(i);
						for (int i=0; i<2; i++) SendLines(i);
					}


					/* Check critical lines */
					for (int y=0; y<4; y++) {
						for (int x=0; (x<XSIZE && winner==-1); x++) {
							if (cmap[y][x]!=0) {
								winner=0;
								SendWin(0);
							}
						}
					}
					for (int y=YSIZE-4; y<YSIZE; y++) {
						for (int x=0; (x<XSIZE && winner==-1); x++) {
							if (cmap[y][x]!=0) {
								winner=1;
								SendWin(1);
							}
						}
					}
					/* Check enemy blocks */
					for (int i=0; i<5; i++) {
						if (players[0].blocks[i]!=NULL) {
							if (players[0].blocks[i]->GetMinY()<=4) {
								winner=0;
								SendWin(0);
							}
						}
					}
					for (int i=0; i<5; i++) {
						if (players[1].blocks[i]!=NULL) {
							if (players[1].blocks[i]->GetMaxY()>=YSIZE-4) {
								winner=1;
								SendWin(1);
							}
						}
					}


	
	
					// only if something has changed //
					if (updateblockmap) SendUpdatedBlockmap(); // NOTICE
				}
	
	
			}
	
		}

	}

	// it will never get here anyways ;)

	cout << "Stopped listening.\n";
	SDLNet_UDP_Close(sock);


}

void server::NewGame() {
	memset(blocks, 0, XSIZE*YSIZE);

	for (int j=0; j<2; j++) for (int i=0; i<5; i++) players[j].blocks[i]=NULL;
	newblocktime=Config.newblocktime;
	gametime=Min2Frame(Config.gametime);
	timestep=1;

	winner=-1;
	noshoot=0;
	pingtimetimer=Config.pingtimerate;
	timeupdate=Config.timeupdaterate;

	char newgame[]="NEWGAME";

	for (int i=0; i<2; i++) {
		for (int j=0; j<5; j++) {
			if (players[i].blocks[j]!=NULL) {
				DeletePlayersBlock(i, j);
			}
		}

		for (int j=0; j<5; j++) {
			players[i].slot[j]=(int)(rand() / (RAND_MAX/7))+1;
		}

		int x = (int)(XSIZE/2)-2;
		int y = (1-i)*YSIZE-(1-i)*4; // 0 for p2, YSIZE-4 for p1

		int type = players[i].slot[ 0 ];
		players[i].blocks[0] = new tetrablock(x, y, type, (char*)blocks );
		players[i].controlling=0;

		players[i].launchdelay=0;
		players[i].movedelay=0;
		players[i].newgame=0;
		players[i].angle=0;
		players[i].loadtime=0;
		players[i].shootwait=0;
		players[i].canshoot=0;
		players[i].score=0;
		players[i].lines=0;
		players[i].SetSelection(0);

		if (players[i].free==false) net::SendUDPpacket(sock, i, newgame);
	}


	if (Config.followers) {
		// informing followers
		for (int j=0; j<MAXFOLLOWERS; j++) {
			if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, newgame);
		}
	}


}

void server::SendEvent(int player, int event) {
	char data[32];
	sprintf (data, "PLAYSOUND %d\n", event);
	net::SendUDPpacket(sock, player, data);

	if (Config.followers) {
		// informing followers
		if (player==0) {
			for (int j=0; j<MAXFOLLOWERS; j++) {
				if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
			}
		}
	}

}

void server::SendCanShoot(int i) {
	char data[200];
	char data2[200];

	if (players[i].shootwait<=0) {
		sprintf (data, "YOUCANSHOOT");
		net::SendUDPpacket(sock, i, data);

		sprintf (data2, "ENEMYCANSHOOT");
		net::SendUDPpacket(sock, 1-i, data2);

		if (Config.followers) {
			// informing followers
			if (i==0) {
				for (int j=0; j<MAXFOLLOWERS; j++) {
					if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
				}
			} else {
				for (int j=0; j<MAXFOLLOWERS; j++) {
					if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data2);
				}
			}
		}

	} else {
		sprintf (data, "YOUNOSHOOT");
		net::SendUDPpacket(sock, i, data);

		sprintf (data2, "ENEMYNOSHOOT");
		net::SendUDPpacket(sock, 1-i, data2);


		if (Config.followers) {
			// informing followers
			if (i==0) {
				for (int j=0; j<MAXFOLLOWERS; j++) {
					if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
				}
			} else {
				for (int j=0; j<MAXFOLLOWERS; j++) {
					if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data2);
				}
			}
		}
	}
}

void server::SendWin(int i) {
	if (i>=0) {
		char data[200];
		char data2[200];
		sprintf (data, "YOUWON");
		net::SendUDPpacket(sock, i, data);
		sprintf (data2, "YOULOST");
		net::SendUDPpacket(sock, 1-i, data2);

		if (Config.followers) {
			// informing followers
			if (i==0) {
				for (int j=0; j<MAXFOLLOWERS; j++) {
					if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
				}
			} else {
				for (int j=0; j<MAXFOLLOWERS; j++) {
					if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data2);
				}
			}
		}

	}
}

void server::SendTime(int i) {
	char data[200];
	sprintf (data, "TIMELEFT %d %d", gametime, timestep);
	net::SendUDPpacket(sock, i, data);

	if (Config.followers) {
		if (i==0) {
			for (int j=0; j<MAXFOLLOWERS; j++) {
				if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
			}
		}
	}
}


void server::SendWantNewgame(int i) {
	char data[200];
	char data2[200];

	if (players[i].newgame>0) {
		sprintf (data, "YOUNEWGAME");
		net::SendUDPpacket(sock, i, data);

		sprintf (data2, "ENEMYNEWGAME");
		net::SendUDPpacket(sock, 1-i, data2);

		if (Config.followers) {
			// informing followers
			if (i==0) {
				for (int j=0; j<MAXFOLLOWERS; j++) {
					if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
				}
			} else {
				for (int j=0; j<MAXFOLLOWERS; j++) {
					if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data2);
				}
			}
		}


	} else {
		sprintf (data, "YOUNONEWGAME");
		net::SendUDPpacket(sock, i, data);

		sprintf (data2, "ENEMYNONEWGAME");
		net::SendUDPpacket(sock, 1-i, data2);

		if (Config.followers) {
			// informing followers
			if (i==0) {
				for (int j=0; j<MAXFOLLOWERS; j++) {
					if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
				}
			} else {
				for (int j=0; j<MAXFOLLOWERS; j++) {
					if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data2);
				}
			}
		}
	}
}


void server::SendEverything() {
	char data[200];
	char data2[200];

	for (int i=0; i<2; i++) {
		char youtwo[]="YOUARETWO";
		net::SendUDPpacket(sock, i, youtwo);

		if (Config.followers) {
			// informing followers
			if (i==0) {
				for (int j=0; j<MAXFOLLOWERS; j++) {
					if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, youtwo);
				}
			}
		}

	}

	for (int i=0; i<2; i++) {
		char enemynick[35];
		sprintf (enemynick, "ENEMYNICK %s", players[1-i].GetNick() );
		net::SendUDPpacket(sock, i, enemynick);

		if (Config.followers) {
			// informing followers
			if (i==0) {
				for (int j=0; j<MAXFOLLOWERS; j++) {
					if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, enemynick);
				}
				char playernick[35];
				sprintf (playernick, "YOURNICK %s", players[i].GetNick() );
				for (int j=0; j<MAXFOLLOWERS; j++) {
					if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, playernick);
				}
			}
		}

	}

	for (int i=0; i<2; i++) SendCanShoot(i);

	for (int i=0; i<2; i++) {
		for (int j=0; j<5; j++) {
			if (players[i].blocks[j]!=NULL) SendUpdatedPlayersBlock(i, j);
			else {
				sprintf (data, "YOURBLOCKDELETED %d", j);
				net::SendUDPpacket(sock, i, data);
				sprintf (data2, "ENEMYBLOCKDELETED %d", j);
				net::SendUDPpacket(sock, 1-i, data2);

				if (Config.followers) {
					// informing followers
					if (i==0) {
						for (int k=0; k<MAXFOLLOWERS; k++) {
							if (followers[k].free==false) net::SendUDPpacket(sock, 2+k, data);
						}
					} else {
						for (int k=0; k<MAXFOLLOWERS; k++) {
							if (followers[k].free==false) net::SendUDPpacket(sock, 2+k, data2);
						}
					}
				}


			}
		}
	}

	SendUpdatedBlockmap();

	for (int i=0; i<2; i++) SendWantNewgame(i);
	for (int i=0; i<2; i++) SendTime(i);
	for (int i=0; i<2; i++) SendPing(i);
	for (int i=0; i<2; i++) SendLines(i);
	for (int i=0; i<2; i++) SendScores(i);
	for (int i=0; i<2; i++) SendUpdatedPlayersSlots(i);

	SendWin(winner);
}


bool server::CheckNickname(char *nick) {
	int len=strlen(nick);

	if (len > NICKMAX) return false;

	for (int i=0; i<len; i++) {
		if (nick[i]<'!' || nick[i]>'z') return false;
	}
	return true;
}

int server::FirstFreePlayerslot() {
	int freeone=-1;
	for (int i=0; i<2; i++) {
		if (players[i].free==true) {
			freeone=i;
			break;
		}
	}
	return freeone;
}

int server::FirstFreeFollowerslot() {
	int freeone=-1;
	for (int i=0; i<MAXFOLLOWERS; i++) {
		if (followers[i].free==true) {
			freeone=i;
			break;
		}
	}
	return freeone;
}



int server::RecognizePlayer(IPaddress address) {
	for (int i=0; i<2; i++) {
		if (address.host==players[i].address.host && address.port == players[i].address.port && players[i].free==false) 
			return i;
	}
	return -1;
}

int server::RecognizeFollower(IPaddress address) {
	for (int i=0; i<2; i++) {
		if (address.host==followers[i].address.host && address.port == followers[i].address.port && 
			followers[i].free==false) return i;
	}
	return -1;
}

int server::PlayersJoined() {
	int num=0;
	for (int i=0; i<2; i++) {
		if (players[i].free==false) num++;
	}
	return num;
}

void server::SendUpdatedPlayersBlock(int player, int i) {
	int x = players[player].blocks[i]->GetX();
	int y = players[player].blocks[i]->GetY();
	int type = players[player].blocks[i]->GetType();
	int angle = players[player].blocks[i]->GetAngle();

	char data[100];
	char data2[100];

	sprintf (data, "YOURBLOCK %d %d %d %d %d", i, x, y, type, angle);
	net::SendUDPpacket(sock, player, data);
	sprintf (data2, "ENEMYBLOCK %d %d %d %d %d", i, x, y, type, angle);
	net::SendUDPpacket(sock, 1-player, data2);

	if (Config.followers) {
		// informing followers
		if (player==0) {
			for (int j=0; j<MAXFOLLOWERS; j++) {
				if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
			}
		} else {
			for (int j=0; j<MAXFOLLOWERS; j++) {
				if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data2);
			}
		}
	}


}

void server::DeletePlayersBlock(int player, int i) {
	delete players[player].blocks[ i ];
	players[player].blocks[ i ]=NULL;

	char data[50];
	char data2[50];

	if (players[player].free==false) {
		sprintf (data, "YOURBLOCKDELETED %d", i);
		net::SendUDPpacket(sock, player, data);
	}
	if (players[1-player].free==false) {
		sprintf (data2, "ENEMYBLOCKDELETED %d", i);
		net::SendUDPpacket(sock, 1-player, data2);
	}

	if (Config.followers) {
		// informing followers
		if (player==0) {
			for (int j=0; j<MAXFOLLOWERS; j++) {
				if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
			}
		} else {
			for (int j=0; j<MAXFOLLOWERS; j++) {
				if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data2);
			}
		}
	}


}

void server::SendUpdatedPlayersSlots(int i) {
	char data[200];
	char data2[200];

	int ptype=0;

	if (players[i].controlling >= 0) {
		if (players[i].blocks[players[i].controlling]!=NULL) ptype = players[i].blocks[players[i].controlling]->GetType();
	}

	sprintf (data, "YOURSLOTS %d %d %d %d %d %d %d", players[i].slot[0], players[i].slot[1], players[i].slot[2],
				players[i].slot[3], players[i].slot[4], players[i].GetSelection(), ptype);
	net::SendUDPpacket(sock, i, data);



	int etype=0;
	if (players[1-i].controlling >= 0) {
		if (players[1-i].blocks[players[1-i].controlling]!=NULL) etype = players[1-i].blocks[players[1-i].controlling]->GetType();
	}

	sprintf (data2, "ENEMYSLOTS %d %d %d %d %d %d %d", players[1-i].slot[0], players[1-i].slot[1], players[1-i].slot[2],
				players[1-i].slot[3], players[1-i].slot[4], players[1-i].GetSelection(), etype);
	net::SendUDPpacket(sock, i, data2);


	if (Config.followers) {
		// informing followers
		if (i==0) {
			for (int j=0; j<MAXFOLLOWERS; j++) {
				if (followers[j].free==false) {
					net::SendUDPpacket(sock, 2+j, data);
					net::SendUDPpacket(sock, 2+j, data2);
				}
			}
		}
	}


}

void server::SendScores(int i) {
	char data[100];
   char data2[100];
	sprintf (data, "YOURSCORE %d", players[i].score);
	net::SendUDPpacket(sock, i, data);

	sprintf (data2, "ENEMYSCORE %d", players[i].score);
	net::SendUDPpacket(sock, 1-i, data2);

	if (Config.followers) {
		// informing followers
		if (i==0) {
			for (int j=0; j<MAXFOLLOWERS; j++) {
				if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
			}
		} else {
			for (int j=0; j<MAXFOLLOWERS; j++) {
				if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data2);
			}
		}
	}

}

void server::SendLines(int i) {
	char data[100];
	char data2[100];

	sprintf (data, "YOURLINES %d", players[i].lines);
	net::SendUDPpacket(sock, i, data);

	sprintf (data2, "ENEMYLINES %d", players[i].lines);
	net::SendUDPpacket(sock, 1-i, data2);

	if (Config.followers) {
		// informing followers
		if (i==0) {
			for (int j=0; j<MAXFOLLOWERS; j++) {
				if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
			}
		} else {
			for (int j=0; j<MAXFOLLOWERS; j++) {
				if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data2);
			}
		}
	}

}

void server::SendPing(int i) {
	char data[512];

	int p1ping = players[i].ping;
	int p2ping = players[1-i].ping;

//	if (players[1-i].free==true) p2ping=0;

	sprintf (data, "PINGTIME %d %d", p1ping, p2ping);
	net::SendUDPpacket(sock, i, data);


	if (Config.followers) {
		// informing followers
		if (i==0) {
			for (int j=0; j<MAXFOLLOWERS; j++) {
				if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
			}
		}
	}

}

void server::SendUpdatedBlockmap() {
	char data[XSIZE*YSIZE+11];
	sprintf(data, "BLOCKMAP ");
	int i=9;
	for (int y=0; y<YSIZE; y++) {
		for (int x=0; x<XSIZE; x++) {
			data[i]=blocks[y][x]+'0';
			i++;
		}
	}
	data[i]='\0';

	for (int k=0; k<2; k++) {
		net::SendUDPpacket(sock, k, data);
	}

	if (Config.followers) {
		// informing followers
		for (int j=0; j<MAXFOLLOWERS; j++) {
			if (followers[j].free==false) net::SendUDPpacket(sock, 2+j, data);
		}
	}

}

Uint32 server::Min2Frame(Uint32 min) {
	return 100*60*min;
}

void server::RawSendAll(char *data) {
	for (int i=0; i<2; i++) net::SendUDPpacket(sock, i, data);

	if (Config.followers) {
		for (int j=0; j<MAXFOLLOWERS; j++) {
			if (followers[j].free==false) net::SendUDPpacket(sock, j+2, data);
		}
	}
}

